      subroutine energyanalysis(nz,n,h,mxo,occ,norbit,qcore,qvale,      &
       nlst,llst,iscore,nmx,r,wfn,itype,coreden,valeden,den,tc,tv,      &
       ccoul,vcoul,cext,vext,cexc,vexc,cvcoul,texc,ec,ev,ecv,etotal,    &
       evale,mxwk,wk)
!  program to calculate the core and valence contributions to the
!   total energy
!   nz = nuclear charge
!   n = number of mesh points
!   h = step size between mesh points ( in bohr)
!   mxo = maximum number of orbitals
!   norbit = actual number of orbital
!   iscore(mxo) = 0 for core state , 1 for valence state
!   llst(mxo) = l for each orbital
!   nlst(mxo) = principal quantum number for each orbital
!   occ(mxo) = occupation of each orbital
!   itype = exchange-correlation type index
!   coreden = core density
!   valeden = valence density
!   den = coreden + valeden
!   tc= core kinetic energy
!   tv= valence kinetic energy
!   ccoul = core coulomb energy
!   vcoul = valence coulomb energy
!   cext = core external energy
!   vext = valence external energy
!   cexc = core exchange-correlation energy
!   vexc = valence exchange-correlation energy
!   cvcoul = core-valence coulomb interaction energy
!   texc = total exchange-correlation energy
!   ec = total core energy
!   ev = total valence energy
!   ecv = total core-valence interaction energy
!   etotal = total energy
!   evale = tv + vcoul + vext + cvcoul + texc - cexc
!   reference: U. von Barth and C. D. Gelatt, Phys. Rev. B 21, 2222(1980)
!
      implicit real*8 (a-h,o-z)
      dimension occ(mxo),wfn(nmx,mxo),den(nmx)
      dimension r(nmx),coreden(nmx),valeden(nmx),wk(nmx,mxwk)
      dimension nlst(mxo),llst(mxo),iscore(mxo)
      noz = 0
      if (mxwk.lt.7) then
       write(6,*) 'error in energyanalysis - mxwk must be gt 7',mxwk
       stop
      endif
      coreden(1:n)=0.d0   !call dcopy (n,0.d0,0,coreden,1)
      valeden(1:n)=0.d0   !call dcopy (n,0.d0,0,valeden,1)
      do io=1,norbit
      if (iscore(io).eq.0) then
         do i=1,n
         coreden(i)=coreden(i)+occ(io)*(wfn(i,io)**2)
         enddo
      endif
      if (iscore(io).eq.1) then
         do i=1,n
         valeden(i)=valeden(i)+occ(io)*(wfn(i,io)**2)
         enddo
      endif
      enddo
      do i=1,n
       den(i)=coreden(i)+valeden(i)
      enddo
!
      etotal=0
      ec=0
      ev=0
      ecv=0
! kinetic energy
      tc=0
      tv=0
      do io=1,norbit
      if (iscore(io).eq.0) then
      call kinetic1(n,h,llst(io),r,wfn(1,io),wfn(1,io),wk,wk(1,2),      &
         wk(1,3),ekin)
!     call kinetic1(n,h,llst(io),r,wfn(1,io),wfn(1,io),wk,wk(1,2),
!    x   wk(1,3),tsum1)
!     write(6,*) 'in energyanal test kin--',io,ekin,tsum1
        tc=tc+occ(io)*ekin
        endif
      if (iscore(io).eq.1) then
      call kinetic1(n,h,llst(io),r,wfn(1,io),wfn(1,io),wk,wk(1,2),      &
         wk(1,3),ekin)
!     call kinetic1(n,h,llst(io),r,wfn(1,io),wfn(1,io),wk,wk(1,2),
!    x   wk(1,3),tsum1)
!     write(6,*) 'in energyanal test kin--',io,ekin,tsum1
        tv=tv+occ(io)*ekin
        endif
      enddo
      write(6,*) 'core and valence kinetic energies', tc,tv
! external potential interaction
      do i=1,n
       wk(i,6)=coreden(i)/r(i)
       wk(i,7)=valeden(i)/r(i)
      enddo
      cext=-2*nz*overint(n,h,wk(1,6))
      vext=-2*nz*overint(n,h,wk(1,7))
      write(6,*) 'core and valence external potential energies',cext,vext
!
      qcore=overint(n,h,coreden)
      qvale=overint(n,h,valeden)
      qtot=qcore+qvale
      qchk=overint(n,h,den)
      write(6,*) 'qcore,qvale,qtot,qchk=', qcore,qvale,qtot,qchk
      call poisson(n,h,qcore,coreden,wk,ccoul)
      chkcvcoul1=overlap(n,h,wk(1,7),wk)
      call exchcor(n,h,coreden,wk,dum,cexc,wk(1,2),wk(1,3),wk(1,4),itype)
      call poisson(n,h,qvale,valeden,wk,vcoul)
      chkcvcoul2=overlap(n,h,wk(1,6),wk)
      call exchcor(n,h,valeden,wk,dum,vexc,wk(1,2),wk(1,3),wk(1,4),itype)
      call poisson(n,h,qtot,den,wk,cvcoul)
      call exchcor(n,h,den,wk,dum,texc,wk(1,2),wk(1,3),wk(1,4),itype)
      cvcoul=cvcoul-vcoul-ccoul
      write(6,*) 'core , valence and interaction coulomb energies'
      write(6,*) ccoul,vcoul,cvcoul,chkcvcoul1,chkcvcoul2
      write(6,*) 'core , valence and interaction exc energies'
      write(6,*) cexc,vexc,texc
      ec=tc+ccoul+cext+cexc
      ev=tv+vcoul+vext+vexc
      ecv=cvcoul+texc-cexc-vexc
      etotal=ec+ev+ecv
      evale=tv+vcoul+vext+cvcoul+texc-cexc
      write(6,*) 'ec = ',ec
      write(6,*) 'ev = ',ev
      write(6,*) 'ecv = ', ecv
      write(6,*) 'etotal = ',etotal
      write(6,*) 'evale = ',evale
      return
       end
