      subroutine exchcor(n,h,den,rvxc,etxc,eexc,a,b,c,itype)
!  calculate exchange correlation potentials and energys
!    for density functional theory from electron density
!  den(n) is electron density * (4*pi*r**2)
!  rvxc(n) is returned as vxc * r
!    to the total energy
!  eexc is the total exchange energy (int(den*exc))
!  etxc is eexc - int(den*vxc)
!  a(n), b(n), c(n), d(n) are work arrays
!  if itype=15 (gga case) need to calculate additional functions:
!       gd=abs(grad(density))/density
!       ggd=laplacian(density)/density
!       gggd=grad(density)*grad(abs(grad(density)))/density**2
!       for r<r0 (fixed in parameter statement), replace density by Gaussian
!         form so that Laplacian does not diverge
!  
    Use GlobalConstants
    Use excorpbe   

       implicit real*8 (a-h,o-z)
       parameter (r0=0.01d0)
       dimension den(n),rvxc(n),a(n),b(n),c(n)
       real*8, allocatable ::  tmpr(:),tmpd(:),tmpv(:)
       fpi = 4*pi      !fpi=16*datan(1.d0)
       i0=r0/h
       i0=max0(i0,2)
       r00=i0*h
!
!  Exchange-correlation contribution  exc-vxc
!  
!!!!!!!!!!!!!!!!!!!PBE form!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   If (itype.eq.16) then
     Allocate(tmpr(n+1),tmpd(n+1),tmpv(n+1))
     
       do i=1,n
          tmpr(i+1)=i*h
          tmpd(i+1)=den(i)/(fpi*(i*h*i*h))
       enddo
         tmpr(1)=0
         tmpd(1)=2*tmpd(2)-tmpd(3)

       call radialexcpbe(n+1,tmpr,tmpd,eexc,tmpv)

         write(6,*) 'eexc',eexc
      
       do i=1,n
         rvxc(i)=tmpv(i+1)*tmpr(i+1)
         tmpv(i)=tmpv(i+1)*den(i)
       enddo
       
       etxc=eexc-overint(n,h,tmpv)
     Deallocate(tmpr,tmpd,tmpv)
     return
!!!!!!!!!!!!!!!end PBE form!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   Else   if (itype.lt.14) then
        do i=1,n
        r=i*h
        r2=r*r
        rho=den(i)/(fpi*r2)
        call excor(rho,vxc,exc,.false.,itype)
        a(i)=den(i)*(exc-vxc)
        b(i)=den(i)*exc
        rvxc(i)=r*vxc
        enddo
   Else if (itype.eq.14) then     ! Perdew-Wang LDA
        do i=1,n
        r=i*h
        r2=r*r
        rho=den(i)/(fpi*r2)
        call pwldafunc(rho,exc,vxc)
        a(i)=den(i)*(exc-vxc)
        b(i)=den(i)*exc
        rvxc(i)=r*vxc
        enddo
   Else   if (itype.eq.15) then
        do i=1,n
        r=i*h
        r2=r*r
        a(i)=den(i)/(fpi*r2)
        b(i)=ddlog(a(i))
        enddo
        call nderiv(h,b,c,n)
        call nderiv(h,c,rvxc,n)
! replace values for r<r0 (i<i0) by Gaussian form
        t1=c(i0)/r00
        t2=rvxc(i0)
        bb=-0.75d0*t1+0.25*t2
        cr2=0.125d0*(t2-t1)
        br=bb*r00
        cr3=cr2*r00
        br2=br*r00
        cr4=cr3*r00
        do i=1,i0
        x=i
        x=x/i0
        x2=x*x
        x3=x*x2
        x4=x2*x2
        a(i)=a(i0)*ddexp(-br2*(x2-1.d0)+cr4*(x4-1.d0))
        c(i)=-2*br*x+4*cr3*x3
        rvxc(i)=-2*bb+12*cr2*x2
        enddo
!
        do i=1,n
        r=i*h
! Laplacian term:
        ggd=2*c(i)/r+rvxc(i)+c(i)**2
! double derivative term:
        gggd=c(i)*(rvxc(i)+c(i)**2)*DSIGN(1.d0,c(i))
! gradient term:
        gd=abs(c(i))
        call excorp(a(i),gd,ggd,gggd,vxc,exc,.false.,itype)
        a(i)=den(i)*(exc-vxc)
        b(i)=den(i)*exc
        rvxc(i)=r*vxc
        enddo
  endif
!  calculate exchange-correlation contribution to the potential
!       etxc=dsum(n,a,1)*h
!       eexc=dsum(n,b,1)*h
        etxc=overint(n,h,a)
        eexc=overint(n,h,b)
        write(6,*) 'etxc,eexc = ',etxc,eexc
        return
         end
